#include "rumd_data.h"
#include <cassert>

class rumd_vhc {
public:
  rumd_vhc();
  ~rumd_vhc() {
    FreeArrays();
  }
  void SetVerbose(bool v) { verbose = v; }
  void SetDirectory(const std::string& directory) { this->directory = directory; }
  void Computevhc(unsigned int nBins, int min_dt, unsigned int first_block=0, int last_block=-1);
  unsigned int GetNumTypes() const { return num_types; }
  unsigned int GetNumBins() const { return num_bins; }
  void Writevhc(const std::string& filename);
  unsigned int GetMaxDataIndex() const { return MaxDataIndex; }

  const double* Getvhc(int type1, int type2) const
  {
    if(type1 < 0 || (unsigned int) type1 >= num_types || type2 < 0 || (unsigned int) type2 >= num_types)
      throw RUMD_Error("rumd_vhc","Getvhc","Invalid type index");

    return vhc[type1][type2];
  }
  const double* GetRVals() const { return rVals; }

private:
  rumd_vhc(const rumd_vhc&);
  rumd_vhc& operator=(const rumd_vhc&);
  void CalcSinglevhc(const Conf &C0, const Conf &Ct, unsigned int num_types);
  void Normalizevhc();
  unsigned long int GetLastIndex();
  void AllocateArrays(unsigned int nTypes, unsigned int nBins);
  void FreeArrays();
  void ResetArrays();
  void CalcR(Conf C0, Conf Ct, double R, unsigned int num_types);

  std::string directory;
  Conf C0;
  Conf Ct;
  unsigned long int *Time;
  unsigned int Count;
  double **R;
  int *numParticlesOfType;
  long ***vhcNonNorm;
  double ***vhc;
  unsigned int MaxDataIndex;
  unsigned int num_types;
  unsigned int num_particles;
  unsigned int particlesPerMol;
  unsigned int num_moleculetypes;

  bool verbose;
  unsigned int numFilenameDigits;
  float dt;
  double *rVals;
  unsigned int num_bins;
  float L;
  float dx;
  unsigned int num_part;
  unsigned int MaxIndex;
};
