
#ifndef DIHEDRALPOTENTIAL_H
#define DIHEDRALPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData;

///////////////////////////////////////////////////////
// Abstract Potential class
///////////////////////////////////////////////////////

class DihedralPotential : public Potential{
 public:
  MoleculeData *M;
  unsigned num_dihedrals;
  unsigned num_blocks, num_threads;
  bool EnergyIncludedInParticleSum() const { return true; };
};


// Ryckaert-Bellemans potential
class DihedralRyckaert : public DihedralPotential{
 private:
  DihedralRyckaert(const DihedralRyckaert&);
  DihedralRyckaert& operator=(const DihedralRyckaert&);
  
 public:
  DihedralRyckaert(MoleculeData *);
  float CalcF(bool initialize, bool calc_stresses, bool report_time);
  double GetPotentialEnergy();
};


///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int energy, class Simbox> 
__global__ void Ryckaert( float4 *r, float4 *f, 
			  uint4 *dlist, uint1 *dtype, float *plist, 
			  float *d_epot_dihedral, float *d_dihedrals,
			  unsigned num_dihedrals, Simbox *simbox, float *simboxpointer  );

/////////////////////////////////////////////////////////

__global__ void DihedralResetForces( float4 *f, float4 *w );
__device__ inline void DihedralAtomicFloatAdd(float *address, float val);


#endif
