import string

template_code = """
float %(class_name)s::CalcF( bool initialize, bool calc_stresses, bool report_time ) {



  cudaEvent_t start;
  cudaEvent_t stop;
  float elapsedTimeForce = 0.;


  neighborList.UpdateNBlist(report_time);
  float elapsedTimeNB = neighborList.GetElapsedTimeLastUpdate();


  if(report_time) {
    cudaEventCreate( &start );
    cudaEventCreate( &stop );
    cudaEventRecord( start, 0);
  }


  
  char flags =  (calc_stresses << 3) | (cutoffMethod << 1) | initialize;
  switch(flags) {
"""

# we create a single big switch statement
# and loop through all values of the template parameters.
# Combine the values bitwise to make a single value to test
for calc_stresses in ["0","1"]:
    for cutoff_method in ["NS","SP","SF"]:
        for initialize in ["0","1"]:
            template_code += """
  case (%(STR)s<<3 | %(CM)s << 1 | %(INIT)s ):
    if(testLESB)
      md_calc_f<%(STR)s,%(CM)s,%(INIT)s><<< kp.grid, kp.threads, kp.shared_size >>>( this, particleData->d_r, particleData->d_f, particleData->d_w, particleData->d_sts, particleData->d_misc, testLESB, testLESB->GetDevicePointer(), neighborList.GetNbListPtr(), neighborList.GetMaxNumNbrBlocks(), neighborList.GetCellListPtr(), pp_objectIndex );
    else
      md_calc_f<%(STR)s,%(CM)s,%(INIT)s><<< kp.grid, kp.threads, kp.shared_size >>>( this, particleData->d_r, particleData->d_f, particleData->d_w, particleData->d_sts, particleData->d_misc, testRSB, testRSB->GetDevicePointer(), neighborList.GetNbListPtr(), neighborList.GetMaxNumNbrBlocks(), neighborList.GetCellListPtr(), pp_objectIndex );
    break;
""" % {"STR":calc_stresses, "CM":cutoff_method, "INIT":initialize}

template_code += """
  }

  if(report_time) {
    cudaEventRecord( stop, 0 );
    cudaEventSynchronize( stop );
    cudaEventElapsedTime( &elapsedTimeForce, start, stop);
    cudaEventDestroy(start);
    cudaEventDestroy(stop);
  }
  return elapsedTimeNB+elapsedTimeForce;
}
"""

# similarly for the function LaunchPE_Kernel

template_code += """
void %(class_name)s::LaunchPE_Kernel() {
  switch( cutoffMethod ){
"""
for cutoff_method in ["NS","SP","SF"]:
    template_code += """
  case %(CM)s:
    if(testLESB)
      md_calc_f<0,%(CM)s,1><<< kp.grid, kp.threads, kp.shared_size >>>( this, particleData->d_r, d_f_pe, particleData->d_w, particleData->d_sts, particleData->d_misc, testLESB, testLESB->GetDevicePointer(), neighborList.GetNbListPtr(), neighborList.GetMaxNumNbrBlocks(), neighborList.GetCellListPtr(), pp_objectIndex );
    else
      md_calc_f<0,%(CM)s,1><<< kp.grid, kp.threads, kp.shared_size >>>( this, particleData->d_r, d_f_pe, particleData->d_w, particleData->d_sts, particleData->d_misc, testRSB, testRSB->GetDevicePointer(), neighborList.GetNbListPtr(), neighborList.GetMaxNumNbrBlocks(), neighborList.GetCellListPtr(), pp_objectIndex );
    break;
    """ % {"CM":cutoff_method}

template_code += """
  }
}
"""

# look in PairPotentials.h to get all the class names

dot_h_file = open("../include/rumd/PairPotential.h")
classNames = []
nextLine = dot_h_file.readline()
while nextLine:
    items = string.split(nextLine)
    # find lines which begin a new PairPotential class (but not the base class)
    if len(items) > 3 and items[0] == "class":
        clsName = items[1]
        if clsName.endswith(":"):
            clsName = clsName[:-1]
        if clsName != "PairPotential":
            classNames.append(clsName)
    nextLine = dot_h_file.readline()

outfile = open("PairPotentialFunctionBodies.inc","w")
outfile.write("""
///////////////////////////////////////////////////////////////////////////////
// This file has been generated by Generate_PP_FunctionBodies.py, do not edit!
///////////////////////////////////////////////////////////////////////////////
""")

for item in classNames:
    outfile.write( template_code % {"class_name":item} )
outfile.close()
