#include "rumd/HypervirialCalculator.h"
#include "rumd/Sample.h"
#include "rumd/Potential.h"

HypervirialCalculator::HypervirialCalculator(Sample* S, float set_delta_ln_rho) : mainSample(S), delta_ln_rho(set_delta_ln_rho) {

  mySample = new Sample(*mainSample);
  mySample->SetVerbose(false);

  // Need to copy the potential(s) from the given sample rather than pass them
  // in 

  std::vector<Potential*>* potentialList = mainSample->GetPotentials();
  std::vector<Potential*>::iterator potIter;
  for(potIter = potentialList->begin(); potIter != potentialList->end(); potIter++) {
    Potential* pot_copy = (*potIter)->Copy();
    my_potentialList.push_back( pot_copy ); // so can delete them later
    mySample->AddPotential( pot_copy );
  }
}


HypervirialCalculator::~HypervirialCalculator() {
  delete mySample;

  std::vector<Potential*>::iterator potIter;
  for(potIter = my_potentialList.begin(); potIter != my_potentialList.end(); potIter++)
    delete (*potIter);
  
}

void HypervirialCalculator::GetDataInfo(std::map<std::string, bool> &writingItem, std::map<std::string, std::string> &columnIDs) {
  writingItem["approx_vir"] = true;
  columnIDs["approx_vir"] = "approx_vir";

  writingItem["approx_h_vir"] = true;
  columnIDs["approx_h_vir"] = "approx_h_vir";
}

void HypervirialCalculator::RemoveDataInfo(std::map<std::string, bool> &writingItem, std::map<std::string, std::string> &columnIDs) {
  writingItem.erase(myPotential->GetID_String());
  columnIDs.erase(myPotential->GetID_String());
}

void HypervirialCalculator::GetDataValues(std::map<std::string, float> &dataValues) {
  
  mainSample->CalcF();
  float main_pe = mainSample->GetPotentialEnergy();
  float main_vir = mainSample->GetVirial();

  (*mySample) = (*mainSample); // copies the data
  float scale_factor = exp(-delta_ln_rho/3.);

  mySample->IsotropicScaleSystem( scale_factor );
  mySample->CalcF();
  float my_pe = mySample->GetPotentialEnergy();
  float my_vir = mySample->GetVirial();
  mySample->IsotropicScaleSystem( 1.f/scale_factor );

  unsigned int nParticles = mySample->GetNumberOfParticles();
  

  dataValues["approx_vir"] = ( my_pe - main_pe )/delta_ln_rho/nParticles;
  //main_pe * ( log(my_pe) - log(main_pe) )/delta_ln_rho/nParticles;
  
  dataValues["approx_h_vir"] = ( my_vir - main_vir )/delta_ln_rho/nParticles;
  //main_vir * ( log(my_vir) - log(main_vir) )/delta_ln_rho/nParticles;

}
